@@ -87,8 +87,9 @@ use strict;
 
 use Carp qw(croak);
 use MIME::Base64 2.21 qw(decode_base64);
+use Module::Runtime qw(use_module);
 
-our $VERSION = "0.002";
+our $VERSION = "0.003";
 
 =head1 CONSTRUCTORS
 
@@ -110,27 +111,12 @@ Known scheme identifiers:
 
 =over
 
-=cut
-
-my %crypt_scheme_handler;
-
 =item B<$1$>
 
 A baroque passphrase scheme based on MD5, designed by
 Poul-Henning Kamp and originally implemented in FreeBSD.  See
 L<Authen::Passphrase::MD5Crypt>.
 
-=cut
-
-$crypt_scheme_handler{"1"} = sub($) {
-	my($passwd) = @_;
-	$passwd =~ m#\A\$1\$([!-~]{0,8})\$([./0-9A-Za-z]{22})\z#
-		or croak "malformed \$1\$ data";
-	require Authen::Passphrase::MD5Crypt;
-	return Authen::Passphrase::MD5Crypt
-		->new(salt => $1, hash_base64 => $2);
-};
-
 =item B<$2$>
 
 =item B<$2a$>
@@ -139,36 +125,11 @@ Two versions of a passphrase scheme based on Blowfish,
 designed by Niels Provos and David Mazieres for OpenBSD.  See
 L<Authen::Passphrase::BlowfishCrypt>.
 
-=cut
-
-$crypt_scheme_handler{"2"} = sub($) {
-	my($passwd) = @_;
-	$passwd =~ m#\A\$2(a?)\$([0-9]{2})\$
-			([./A-Za-z0-9]{22})([./A-Za-z0-9]{31})\z#x
-		or croak "malformed \$2\$ data";
-	require Authen::Passphrase::BlowfishCrypt;
-	return Authen::Passphrase::BlowfishCrypt->new(
-			key_nul => $1, cost => $2,
-			salt_base64 => $3, hash_base64 => $4);
-};
-
-$crypt_scheme_handler{"2a"} = $crypt_scheme_handler{"2"};
-
 =item B<$3$>
 
 The NT-Hash scheme, which stores the MD4 hash of the passphrase expressed
 in Unicode.  See L<Authen::Passphrase::NTHash>.
 
-=cut
-
-$crypt_scheme_handler{"3"} = sub($) {
-	my($passwd) = @_;
-	$passwd =~ m#\A\$3\$\$([0-9a-f]{32})\z#
-		or croak "malformed \$3\$ data";
-	require Authen::Passphrase::NTHash;
-	return Authen::Passphrase::NTHash->new(hash_hex => $1);
-};
-
 =item B<$IPB2$>
 
 Invision Power Board 2.x salted MD5
@@ -184,16 +145,6 @@ halves of a LAN Manager hash can be separated and manipulated
 independently; this represents such an isolated half.  See
 L<Authen::Passphrase::LANManagerHalf>.
 
-=cut
-
-$crypt_scheme_handler{"LM"} = sub($) {
-	my($passwd) = @_;
-	$passwd =~ m#\A\$LM\$([0-9a-f]{16})\z#
-		or croak "malformed \$LM\$ data";
-	require Authen::Passphrase::LANManagerHalf;
-	return Authen::Passphrase::LANManagerHalf->new(hash_hex => $1);
-};
-
 =item B<$NT$>
 
 The NT-Hash scheme, which stores the MD4 hash of the passphrase expressed
@@ -202,19 +153,10 @@ in Unicode.  See L<Authen::Passphrase::NTHash>.
 The B<$3$> identifier refers to the same hash algorithm, but has a
 slightly different textual format (an extra "B<$>").
 
-=cut
-
-$crypt_scheme_handler{"NT"} = sub($) {
-	my($passwd) = @_;
-	$passwd =~ m#\A\$NT\$([0-9a-f]{32})\z#
-		or croak "malformed \$NT\$ data";
-	require Authen::Passphrase::NTHash;
-	return Authen::Passphrase::NTHash->new(hash_hex => $1);
-};
-
 =item B<$P$>
 
-Portable PHP password hash (see L<http://www.openwall.com/phpass/>)
+Portable PHP password hash (phpass), based on MD5.  See
+L<Authen::Passphrase::PHPass>.
 
 =item B<$VMS1$>
 
@@ -223,6 +165,7 @@ Portable PHP password hash (see L<http://www.openwall.com/phpass/>)
 =item B<$VMS3$>
 
 Three variants of the Purdy polynomial system used in VMS.
+See L<Authen::Passphrase::VMSPurdy>.
 
 =item B<$af$>
 
@@ -276,36 +219,49 @@ are regarded as invalid encodings.
 
 =cut
 
+my %crypt_scheme_handler = (
+	"1"    => [ "Authen::Passphrase::MD5Crypt", 0.003 ],
+	"2"    => [ "Authen::Passphrase::BlowfishCrypt", 0.003 ],
+	"2a"   => [ "Authen::Passphrase::BlowfishCrypt", 0.003 ],
+	"3"    => [ "Authen::Passphrase::NTHash", 0.003 ],
+	"IPB2" => sub($) { croak '$IPB2$ is unimplemented' },
+	"K4"   => sub($) { croak '$K4$ is unimplemented' },
+	"LM"   => [ "Authen::Passphrase::LANManagerHalf", 0.003 ],
+	"NT"   => [ "Authen::Passphrase::NTHash", 0.003 ],
+	"P"    => [ "Authen::Passphrase::PHPass", 0.003 ],
+	"VMS1" => [ "Authen::Passphrase::VMSPurdy", 0.003 ],
+	"VMS2" => [ "Authen::Passphrase::VMSPurdy", 0.003 ],
+	"VMS3" => [ "Authen::Passphrase::VMSPurdy", 0.003 ],
+	"af"   => sub($) { croak '$af$ is unimplemented' },
+	"apr1" => sub($) { croak '$apr1$ is unimplemented' },
+	"krb5" => sub($) { croak '$krb5$ is unimplemented' },
+);
+
 sub from_crypt($$) {
 	my($class, $passwd) = @_;
-	croak "from_crypt constructor called on $class instead of ".__PACKAGE__
+	croak "crypt string \"$passwd\" not supported for $class"
 		unless $class eq __PACKAGE__;
-	croak "invalid character in crypt string" if $passwd =~ /[^!-9;-~]/;
+	my $handler;
 	if($passwd =~ /\A\$([0-9A-Za-z]+)\$/) {
 		my $scheme = $1;
-		my $handler = $crypt_scheme_handler{$scheme};
+		$handler = $crypt_scheme_handler{$scheme};
 		croak "unrecognised crypt scheme \$$scheme\$"
 			unless defined $handler;
-		return $handler->($passwd);
-	} elsif($passwd =~ m#\A([./0-9A-Za-z]{2})([./0-9A-Za-z]{11})\z#) {
-		require Authen::Passphrase::DESCrypt;
-		return Authen::Passphrase::DESCrypt
-				->new(salt_base64 => $1, hash_base64 => $2);
-	} elsif($passwd =~ m#\A_([./0-9A-Za-z]{4})([./0-9A-Za-z]{4})
-				([./0-9A-Za-z]{11})\z#x) {
-		require Authen::Passphrase::DESCrypt;
-		return Authen::Passphrase::DESCrypt
-				->new(fold => 1, nrounds_base64 => $1,
-				      salt_base64 => $2, hash_base64 => $3);
+	} elsif($passwd =~ m#\A(?:[^\$].{12}|_.{19})\z#s) {
+		$handler = [ "Authen::Passphrase::DESCrypt", 0.003 ];
 	} elsif($passwd eq "") {
-		require Authen::Passphrase::AcceptAll;
-		return Authen::Passphrase::AcceptAll->new;
-	} elsif(length($passwd) < 13 && $passwd !~ /\A\$/) {
-		require Authen::Passphrase::RejectAll;
-		return Authen::Passphrase::RejectAll->new;
+		$handler = [ "Authen::Passphrase::AcceptAll", 0.003 ];
+	} elsif($passwd =~ /\A[^\$].{0,11}\z/s) {
+		$handler = [ "Authen::Passphrase::RejectAll", 0.003 ];
 	} else {
 		croak "bad crypt syntax in \"$passwd\"";
 	}
+	if(ref($handler) eq "CODE") {
+		return $handler->($passwd);
+	} else {
+		my($modname, $modver) = @$handler;
+		return use_module($modname, $modver)->from_crypt($passwd);
+	}
 }
 
 =item Authen::Passphrase->from_rfc2307(USERPASSWORD)
@@ -321,41 +277,21 @@ Known scheme identifiers:
 
 =over
 
-=cut
-
-my %rfc2307_scheme_handler;
-
-my $rfc2307_placeholder = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ /\A\{(.*?)\}/;
-	my $scheme = uc($1);
-	croak "{$scheme} is a placeholder";
-};
-
 =item B<{CLEARTEXT}>
 
 Passphrase stored in cleartext.  See L<Authen::Passphrase::Clear>.
 
-=cut
-
-$rfc2307_scheme_handler{CLEARTEXT} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ s/\A\{.*?\}//;
-	require Authen::Passphrase::Clear;
-	return Authen::Passphrase::Clear->new($userpassword);
-};
-
 =item B<{CRYPT}>
 
-Any crypt encoding.
+The scheme identifier is followed by a crypt string.
 
-=cut
+=item B<{CRYPT16}>
 
-$rfc2307_scheme_handler{CRYPT} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ s/\A\{.*?\}//;
-	return Authen::Passphrase->from_crypt($userpassword);
-};
+Used ambiguously by Exim, to refer to either crypt16
+(see L<Authen::Passphrase::Crypt16>) or bigcrypt (see
+L<Authen::Passphrase::BigCrypt>) depending on compilation options.
+This is a bug, resulting from a confusion between the two algorithms.
+This module does not support any meaning for this scheme identifier.
 
 =item B<{K5KEY}>
 
@@ -363,10 +299,6 @@ Not a real passphrase scheme, but a placeholder to indicate that a
 Kerberos key stored separately should be checked against.  No data
 follows the scheme identifier.
 
-=cut
-
-$rfc2307_scheme_handler{K5KEY} = $rfc2307_placeholder;
-
 =item B<{KERBEROS}>
 
 Not a real passphrase scheme, but a placeholder to indicate that
@@ -374,206 +306,114 @@ Kerberos should be invoked to check against a user's passphrase.
 The scheme identifier is followed by the user's username, in the form
 "I<name>B<@>I<realm>".
 
-=cut
-
-$rfc2307_scheme_handler{KERBEROS} = $rfc2307_placeholder;
-
 =item B<{LANM}>
 
 Synonym for B<{LANMAN}>, used by CommuniGate Pro.
 
-=cut
-
-$rfc2307_scheme_handler{"LANM"} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ /\A\{.*?\}([0-9a-fA-F]{32})\z/
-		or croak "malformed {LANMAN} data";
-	require Authen::Passphrase::LANManager;
-	return Authen::Passphrase::LANManager->new(hash_hex => $1);
-};
-
 =item B<{LANMAN}>
 
 The Microsoft LAN Manager hash scheme.  See
 L<Authen::Passphrase::LANManager>.
 
-=cut
-
-$rfc2307_scheme_handler{"LANMAN"} = $rfc2307_scheme_handler{"LANM"};
-
 =item B<{MD4}>
 
 The MD4 digest of the passphrase is stored.  See
 L<Authen::Passphrase::SaltedDigest>.
 
-=cut
-
-$rfc2307_scheme_handler{MD4} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ s/\A\{.*?\}//;
-	require Authen::Passphrase::SaltedDigest;
-	return Authen::Passphrase::SaltedDigest->new(algorithm => "MD4",
-		hash => decode_base64($userpassword));
-};
-
 =item B<{MD5}>
 
 The MD5 digest of the passphrase is stored.  See
 L<Authen::Passphrase::SaltedDigest>.
 
-=cut
-
-$rfc2307_scheme_handler{MD5} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ s/\A\{.*?\}//;
-	require Authen::Passphrase::SaltedDigest;
-	return Authen::Passphrase::SaltedDigest->new(algorithm => "MD5",
-		hash => decode_base64($userpassword));
-};
-
 =item B<{MSNT}>
 
 The NT-Hash scheme, which stores the MD4 hash of the passphrase expressed
 in Unicode.  See L<Authen::Passphrase::NTHash>.
 
-=cut
-
-$rfc2307_scheme_handler{"MSNT"} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ m#\A\{.*?\}([0-9A-Fa-f]{32})\z#
-		or croak "malformed {MSNT} data";
-	require Authen::Passphrase::NTHash;
-	return Authen::Passphrase::NTHash->new(hash_hex => $1);
-};
-
 =item B<{NS-MTA-MD5}>
 
 An MD5-based scheme used by Netscape Mail Server.  See
 L<Authen::Passphrase::NetscapeMail>.
 
-=cut
-
-$rfc2307_scheme_handler{"NS-MTA-MD5"} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ /\A\{.*?\}([0-9a-f]{32})([!-~]{32})\z/
-		or croak "malformed {NS-MTA-MD5} data";
-	require Authen::Passphrase::NetscapeMail;
-	return Authen::Passphrase::NetscapeMail
-		->new(salt => $2, hash_hex => $1);
-};
-
 =item B<{RMD160}>
 
 The RIPEMD-160 digest of the passphrase is stored.  See
 L<Authen::Passphrase::SaltedDigest>.
 
-=cut
-
-$rfc2307_scheme_handler{RMD160} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ s/\A\{.*?\}//;
-	require Authen::Passphrase::SaltedDigest;
-	return Authen::Passphrase::SaltedDigest->new(
-		algorithm => "Crypt::RIPEMD160-",
-		hash => decode_base64($userpassword));
-};
-
 =item B<{SASL}>
 
 Not a real passphrase scheme, but a placeholder to indicate that SASL
 should be invoked to check against a user's passphrase.  The scheme
 identifier is followed by the user's username.
 
-=cut
-
-$rfc2307_scheme_handler{SASL} = $rfc2307_placeholder;
-
 =item B<{SHA}>
 
 The SHA-1 digest of the passphrase is stored.  See
 L<Authen::Passphrase::SaltedDigest>.
 
-=cut
-
-$rfc2307_scheme_handler{SHA} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ s/\A\{.*?\}//;
-	require Authen::Passphrase::SaltedDigest;
-	return Authen::Passphrase::SaltedDigest->new(algorithm => "SHA-1",
-		hash => decode_base64($userpassword));
-};
-
 =item B<{SMD5}>
 
 The MD5 digest of the passphrase plus a salt is stored.  See
 L<Authen::Passphrase::SaltedDigest>.
 
-=cut
-
-$rfc2307_scheme_handler{SMD5} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ s/\A\{.*?\}//;
-	my $hash_and_salt = decode_base64($userpassword);
-	croak "not a valid MD5 hash" if length($hash_and_salt) < 16;
-	require Authen::Passphrase::SaltedDigest;
-	return Authen::Passphrase::SaltedDigest->new(algorithm => "MD5",
-		salt => substr($hash_and_salt, 16),
-		hash => substr($hash_and_salt, 0, 16));
-};
-
 =item B<{SSHA}>
 
 The SHA-1 digest of the passphrase plus a salt is stored.
 See L<Authen::Passphrase::SaltedDigest>.
 
-=cut
-
-$rfc2307_scheme_handler{SSHA} = sub($) {
-	my($userpassword) = @_;
-	$userpassword =~ s/\A\{.*?\}//;
-	my $hash_and_salt = decode_base64($userpassword);
-	croak "not a valid SHA-1 hash" if length($hash_and_salt) < 20;
-	require Authen::Passphrase::SaltedDigest;
-	return Authen::Passphrase::SaltedDigest->new(algorithm => "SHA-1",
-		salt => substr($hash_and_salt, 20),
-		hash => substr($hash_and_salt, 0, 20));
-};
-
 =item B<{UNIX}>
 
 Not a real passphrase scheme, but a placeholder to indicate that Unix
 mechanisms should be used to check against a Unix user's login passphrase.
 The scheme identifier is followed by the user's username.
 
-=cut
-
-$rfc2307_scheme_handler{UNIX} = $rfc2307_placeholder;
-
 =item B<{WM-CRY}>
 
 Synonym for B<{CRYPT}>, used by CommuniGate Pro.
 
-=cut
-
-$rfc2307_scheme_handler{"WM-CRY"} = $rfc2307_scheme_handler{CRYPT};
-
 =back
 
 =cut
 
+my %rfc2307_scheme_handler = (
+	"CLEARTEXT"  => [ "Authen::Passphrase::Clear", 0.003 ],
+	# "CRYPT" is handled specially
+	"CRYPT16"    => sub($) { croak "{CRYPT16} is ambiguous" },
+	"K5KEY"      => sub($) { croak "{K5KEY} is a placeholder" },
+	"KERBEROS"   => sub($) { croak "{KERBEROS} is a placeholder" },
+	"LANM"       => [ "Authen::Passphrase::LANManager", 0.003 ],
+	"LANMAN"     => [ "Authen::Passphrase::LANManager", 0.003 ],
+	"MD4"        => [ "Authen::Passphrase::SaltedDigest", 0.003 ],
+	"MD5"        => [ "Authen::Passphrase::SaltedDigest", 0.003 ],
+	"MSNT"       => [ "Authen::Passphrase::NTHash", 0.003 ],
+	"NS-MTA-MD5" => [ "Authen::Passphrase::NetscapeMail", 0.003 ],
+	"RMD160"     => [ "Authen::Passphrase::SaltedDigest", 0.003 ],
+	"SASL"       => sub($) { croak "{SASL} is a placeholder" },
+	"SHA"        => [ "Authen::Passphrase::SaltedDigest", 0.003 ],
+	"SMD5"       => [ "Authen::Passphrase::SaltedDigest", 0.003 ],
+	"SSHA"       => [ "Authen::Passphrase::SaltedDigest", 0.003 ],
+	"UNIX"       => sub($) { croak "{UNIX} is a placeholder" },
+	# "WM-CRY" is handled specially
+);
+
 sub from_rfc2307($$) {
 	my($class, $userpassword) = @_;
-	croak "from_rfc2307 constructor called on $class instead of ".
-			__PACKAGE__
+	return $class->from_crypt($1)
+		if $userpassword =~ m#\A\{(?i:crypt|wm-cry)\}(.*)\z#s;
+	croak "RFC 2307 string \"$userpassword\" not supported for $class"
 		unless $class eq __PACKAGE__;
-	croak "invalid character in RFC 2307 string"
-		if $userpassword =~ /[^!-~]/;
 	$userpassword =~ /\A\{([-0-9a-z]+)\}/i
 		or croak "bad RFC 2307 syntax in \"$userpassword\"";
 	my $scheme = uc($1);
 	my $handler = $rfc2307_scheme_handler{$scheme};
 	croak "unrecognised RFC 2307 scheme {$scheme}" unless defined $handler;
-	return $handler->($userpassword);
+	if(ref($handler) eq "CODE") {
+		return $handler->($userpassword);
+	} else {
+		my($modname, $modver) = @$handler;
+		return use_module($modname, $modver)
+			->from_rfc2307($userpassword);
+	}
 }
 
 =back
@@ -617,13 +457,15 @@ Any subclass must implement the C<match> method.  That is the minimum
 required.
 
 Subclasses should implement the C<as_crypt> and C<as_rfc2307> methods
-wherever possible, with the following exception.  If a passphrase scheme
-has a crypt encoding but no native RFC 2307 encoding, so it can be RFC
-2307 encoded only by using the "B<{CRYPT}>" scheme, then C<as_rfc2307>
-should I<not> be implemented by the class.  There is a default
-implementation of the C<as_rfc_2307> method that uses "B<{CRYPT}>"
-automatically for passphrase schemes that do not have a native RFC
-2307 encoding.
+and the C<from_crypt> and C<from_rfc2307> constructors wherever
+appropriate, with the following exception.  If a passphrase scheme has
+a crypt encoding but no native RFC 2307 encoding, so it can be RFC 2307
+encoded only by using the "B<{CRYPT}>" scheme, then C<as_rfc2307> and
+C<from_rfc2307> should I<not> be implemented by the class.  There is a
+default implementation of the C<as_rfc2307> method that uses "B<{CRYPT}>"
+and C<as_crypt>, and a default implementation of the C<from_rfc2307>
+method that recognises "B<{CRYPT}>" and passes the embedded crypt string
+to the C<from_crypt> constructor.
 
 Implementation of the C<passphrase> method is entirely optional.
 It should be attempted only for schemes that are so ludicrously weak as
